/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.kernel;

/**
 * Exception thrown when an internal error occurs in the Cadmium interpreter
 * or runtime (<i>e.g.</i> a primitive method is missing).
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
public final class CadmiumException extends Exception {

    /** Serialization UID. */
    static final long serialVersionUID = -8390189313774197314L;

    /**
     * Constructs a new exception from message and cause.
     * @param msg exception message
     * @param cause exception cause
     */
    public CadmiumException(final String msg, final Throwable cause) {
        super(msg, cause);
    } // end constructor(String, Throwable)

    /**
     * Constructs a new exception from message.
     * @param msg exception message
     */
    public CadmiumException(final String msg) {
        super(msg);
    } // end constructor(String)

    /**
     * Constructs a new exception from cause.
     * @param cause exception cause
     */
    public CadmiumException(final Throwable cause) {
        super(cause);
    } // end constructor(Throwable)

} // end class 'CadmiumException'
