/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.kernel;

import java.io.InputStream;
import java.net.URL;

/**
 * This interface define a hook called when an attempt is made to open an
 * input stream over a file.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
public interface FileHook {

    /**
     * Called when an attempt is made to open an input stream.
     * @param file file to be opened
     * @return an input stream over the passed file,
     *         <tt>null</tt> if no such file can be opened
     */
    InputStream getInputStream(final String file);

    /**
     * Tries to get the URL for a file.
     * @param file file to be opened
     * @return the URL of the file to be opened by {@link #getInputStream(java.lang.String)},
     *         <tt>null</tt> if no such file can be opened
     */
    URL getURL(final String file);

} // end interface 'FileHook'
