/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.cadmiumjdbc;

import java.sql.Savepoint;

import fr.x9c.cadmium.kernel.Block;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Custom;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;
import fr.x9c.cadmium.primitives.cadmium.Cadmium;

/**
 * This class provides the primitives related to the <tt>java.sql.Savepoint</tt> class.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.1
 * @since 1.1
 */
@PrimitiveProvider
public final class Savepoints {

    /**
     * No instance of this class.
     */
    private Savepoints() {
    } // end empty constructor

    /**
     * Returns the identifier of the passed savepoint.
     * @param ctxt context
     * @param savepoint savepoint
     * @return the identifier of the passed savepoint
     * @throws Fail.Exception if any error occurs
     */
    @Primitive
    public static Value cadmiumjdbc_savepoint_get_savepoint_id(final CodeRunner ctxt,
                                                               final Value savepoint)
        throws Fail.Exception {
        try {
            final Savepoint sp = (Savepoint) savepoint.asBlock().asCustom();
            final Block res = Block.createCustom(Custom.INT_32_SIZE,
                                                 Custom.INT_32_OPS);
            res.setInt32(sp.getSavepointId());
            return Value.createFromBlock(res);
        } catch (final Exception e) {
            Cadmium.fail(ctxt, e);
            return Value.UNIT; // never reached
        } // end try/catch
    } // end method 'cadmiumjdbc_savepoint_get_savepoint_id(CodeRunner, Value)'

    /**
     * Returns the name of the passed savepoint.
     * @param ctxt context
     * @param savepoint savepoint
     * @return the name of the passed savepoint
     * @throws Fail.Exception if any error occurs
     */
    @Primitive
    public static Value cadmiumjdbc_savepoint_get_savepoint_name(final CodeRunner ctxt,
                                                                 final Value savepoint)
        throws Fail.Exception {
        try {
            final Savepoint sp = (Savepoint) savepoint.asBlock().asCustom();
            final Block res = Block.createString(sp.getSavepointName());
            return Value.createFromBlock(res);
        } catch (final Exception e) {
            Cadmium.fail(ctxt, e);
            return Value.UNIT; // never reached
        } // end try/catch
    } // end method 'cadmiumjdbc_savepoint_get_savepoint_name(CodeRunner, Value)'

} // end class 'Savepoints'
