/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.dbm;

import java.util.Enumeration;

/**
 * This class is a mere structure describing a (database, enumeration) pair. <br/>
 * In a standard setting, this class would be an inner-class of
 * <tt>fr.x9c.cadmium.primitives.dbm.JdbmImplementation</tt>. However, due to the
 * necessity of dynamic loading of <tt>fr.x9c.cadmium.primitives.dbm.JdbmImplementation</tt>,
 * this class has been made <i>standalone</i> (and public) to avoid unnecessary juggling
 * with class loaders.
 *
 * @param <T> type of database instance
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.1
 * @since 1.1
 */
public final class JdbmImplementationDB<T> {

    /** Actual database. */
    public final T db;

    /** Current enumeration over keys. */
    public Enumeration enumeration;

    /**
     * Constructs a structure from a database. <br/>
     * Enumeration is set to <tt>null</tt>.
     * @param d handle to database - should not be <tt>null</tt>
     */
    public JdbmImplementationDB(final T d) {
        assert d != null : "null d";
        this.db = d;
        this.enumeration = null;
    } // end constructor(T)

} // end class 'JdbmImplementationDB'
