/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.labltk;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Context;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * Implements all primitives from 'cltkCaml.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.3
 * @since 1.3
 */
@PrimitiveProvider
public final class CltkCaml {

    /**
     * No instance of this class.
     */
    private CltkCaml() {
    } // end empty constructor

    /**
     * Sets the return value of the interpreter.
     * @param ctxt context
     * @param v return value for the interpreter
     * @return <i>unit</i>
     */
    @Primitive
    public static Value camltk_return(final CodeRunner ctxt,
                                      final Value v)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        slot.getInterpreter().setResult(v.asBlock().asString());
        return Value.UNIT;
    } // end method 'camltk_return(CodeRunner, Value)'

    /**
     * Initializes the 'labltk' library.
     * @param ctxt context
     * @param unit ignored
     * @return <i>unit</i>
     */
    @Primitive
    public static Value camltk_init(final CodeRunner ctxt,
                                    final Value unit) {
        final Context c = ctxt.getContext();
        final TKSlot slot = (TKSlot) c.getSlot(TKSlot.SLOT);
        if (slot == null) {
            c.registerSlot(TKSlot.SLOT, new TKSlot(ctxt.getContext()));
        } // end if
        return Value.UNIT;
    } // end method 'camltk_init(CodeRunner, Value)'

} // end class 'CltkCaml'
