/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.labltk;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

import tcl.lang.Interp;
import tcl.lang.Notifier;
import tcl.lang.TCL;

/**
 * Implements all primitives from 'cltkEvent.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.3
 * @since 1.3
 */
@PrimitiveProvider
public final class CltkEvent {

    /** Flags for waited events. */
    private static final int[] FLAGS = {
        TCL.DONT_WAIT,
        TCL.WINDOW_EVENTS,
        TCL.FILE_EVENTS,
        TCL.TIMER_EVENTS,
        TCL.IDLE_EVENTS,
        TCL.ALL_EVENTS
    };

    /**
     * No instance of this class.
     */
    private CltkEvent() {
    } // end empty constructor

    /**
     * Executes events until exhaustion.
     * @param ctxt context
     * @param unit ignore
     * @return <i>unit</i>
     */
    @Primitive
    public static Value camltk_tk_mainloop(final CodeRunner ctxt,
                                           final Value unit)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        final Interp interp = slot.getInterpreter();
        final Notifier notifier = interp.getNotifier();
        int result = 1;
        while (result == 1) {
            result = notifier.doOneEvent(TCL.ALL_EVENTS);
        } // end while
        return Value.UNIT;
    } // end method 'camltk_tk_mainloop(CodeRunner, Value)'

    /**
     * Executes one event.
     * @param ctxt context
     * @param flags events to wait for
     * @return whether the event was successfully executed
     */
    @Primitive
    public static Value camltk_dooneevent(final CodeRunner ctxt,
                                          final Value flags)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        final Interp interp = slot.getInterpreter();
        final Notifier notifier = interp.getNotifier();
        final int fl = fr.x9c.cadmium.kernel.Misc.convertFlagList(flags, CltkEvent.FLAGS);
        final int result = notifier.doOneEvent(fl);
        return result == 0 ? Value.FALSE : Value.TRUE;
    } // end method 'camltk_dooneevent(CodeRunner, Value)'

} // end class 'CltkEvent'
