/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.labltk;

import fr.x9c.cadmium.kernel.Block;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

import tcl.lang.Interp;
import tcl.lang.TclList;
import tcl.lang.TclObject;
import tcl.lang.TclString;

/**
 * Implements all primitives from 'cltkMisc.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.3
 * @since 1.3
 */
@PrimitiveProvider
public final class CltkMisc {

    /**
     * No instance of this class.
     */
    private CltkMisc() {
    } // end empty constructor

    /**
     * Splits a string into a list of strings.
     * @param ctxt context
     * @param v string to split
     * @throws Fail.Exception if the passed string is invalid
     */
    @Primitive
    public static Value camltk_splitlist(final CodeRunner ctxt,
                                         final Value v)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        final Interp interp = slot.getInterpreter();
        final TclObject s = TclString.newInstance(v.asBlock().asString());
        try {
            TclList.append(interp, s, TclString.newInstance(""));
            final TclObject[] elems = TclList.getElements(interp, s);
            Value res = Value.EMPTY_LIST;
            for (int i = elems.length - 1 - 1; i >= 0; i--) {
                final Block b = Block.createString(elems[i].toString());
                final Block cons = Block.createBlock(Block.TAG_CONS,
                                                     Value.createFromBlock(b),
                                                     res);
                res = Value.createFromBlock(cons);
            } // end for
            return res;
        } catch (final Throwable t) {
            slot.tkError(t);
            return Value.UNIT; // never reached
        } // end try/catch
    } // end method 'camltk_splitlist(CodeRunner, Value)'

} // end class 'CltkMisc'
