/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.labltk;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

import tcl.lang.TimerHandler;

/**
 * Implements all primitives from 'cltkTimer.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.3
 * @since 1.3
 */
@PrimitiveProvider
public final class CltkTimer {

    /**
     * No instance of this class.
     */
    private CltkTimer() {
    } // end empty constructor

    /**
     * Creates a timer.
     * @param ctxt context
     * @param milli time to before timer exhaustion in milliseconds
     * @param cbid identifier of callback to call on timer exhaustion
     * @return the identifier for the created timer
     */
    @Primitive
    public static Value camltk_add_timer(final CodeRunner ctxt,
                                         final Value milli,
                                         final Value cbid)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        final int res = slot.createTimerHandler(ctxt, milli.asLong(), cbid.asLong());
        return Value.createFromLong(res);
    } // end method 'camltk_add_timer(CodeRunner, Value, Value)'

    /**
     * Cancels a timer.
     * @param ctxt context
     * @param token identifier of timer to cancel
     * @return <i>unit</i>
     */
    @Primitive
    public static Value camltk_rem_timer(final CodeRunner ctxt,
                                         final Value token)
        throws Fail.Exception {
        final TKSlot slot = TKSlot.checkInit(ctxt);
        final TimerHandler th = slot.getTimerHandler(token.asLong());
        if (th != null) {
            th.cancel();
        } // end if
        return Value.UNIT;
    } // end method 'camltk_rem_timer(CodeRunner, Value)'

} // end class 'CltkTimer'
