/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.stdlib;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * Implements all primitives from 'backtrace.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.2
 * @since 1.2
 */
@PrimitiveProvider
public final class BackTrace {

    /**
     * No instance of this class.
     */
    private BackTrace() {
    } // end empty constructor

    /**
     * Sets the backtrace status (true meaning that the backtrace
     * is recorded).
     * @param ctxt context
     * @param flag new backtrace status
     * @return <i>unit</i>
     */
    @Primitive
    public static Value caml_record_backtrace(final CodeRunner ctxt,
                                              final Value flag) {
        ctxt.getContext().setBacktraceActive(flag != Value.FALSE);
        return Value.UNIT;
    } // end method 'caml_record_backtrace(CodeRunner, Value)'

    /**
     * Returns the backtrace status (true meaning that the backtrace
     * is recorded).
     * @param ctxt context
     * @param unit ignored
     * @return whether the backtrace is recorded
     */
    @Primitive
    public static Value caml_backtrace_status(final CodeRunner ctxt,
                                              final Value unit) {
        return ctxt.getContext().isBacktraceActive() ? Value.TRUE : Value.FALSE;
    } // end method 'caml_backtrace_status(CodeRunner, Value)'

    /**
     * Returns the recorded backtrace.
     * @param ctxt context
     * @param unit ignored
     * @return the recorded backtrace
     */
    @Primitive
    public static Value caml_get_exception_backtrace(final CodeRunner ctxt,
                                                     final Value unit) {
        return ctxt.getExceptionBacktrace();
    } // end method 'caml_get_exception_backtrace(CodeRunner, Value)'

} // end class 'BackTrace'
