/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.stdlib;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * Implements all primitives from 'signals.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Signals {

    /**
     * No instance of this class.
     */
    private Signals() {
    } // end empty constructor

    /**
     * Installs a signal handler, returning the current one.
     * @param ctxt context
     * @param signal signal identifier
     * @param action function to be called when signal is intercepted
     * @return the current signal handler for the passed signal
     * @throws Fail.Exception if signal is invalid
     */
    @Primitive
    public static Value caml_install_signal_handler(final CodeRunner ctxt,
                                                    final Value signal,
                                                    final Value action)
        throws Fail.Exception {
        return ctxt.getContext().installSignalHandler(signal, action);
    } // end method 'caml_install_signal_handler(CodeRunner, Value, Value)'

} // end class 'Signals'
