/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.stdlib;

import fr.x9c.cadmium.kernel.ByteCodeRunner;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * Implements all primitives from 'stacks.c'.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Stacks {

    /**
     * No instance of this class.
     */
    private Stacks() {
    } // end empty constructor

    /**
     * Ensures that the stack may contain a given number of values,
     * and grow it if needed.
     * @param ctxt context
     * @param sz minimal size of stack
     * @return <i>unit</i>
     * @throws Fail.Exception if stack cannot be resized
     *                        (<i>e. g.</i> due to available memory)
     */
    @Primitive
    public static Value caml_ensure_stack_capacity(final CodeRunner ctxt,
                                                   final Value sz)
        throws Fail.Exception {
        if (ctxt.isNative() ) {
            ((ByteCodeRunner) ctxt).resizeStack(sz.asLong());
        } // end if
        return Value.UNIT;
    } // end method 'caml_ensure_stack_capacity(CodeRunner, Value)'

} // end class 'Stacks'
