/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.systhreads;

import java.util.concurrent.atomic.AtomicBoolean;

import fr.x9c.cadmium.kernel.Custom;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class defines operations for custom type "_mutex".
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
final class CustomMutex extends Custom.AbstractOperations {

    /** Custom operations for "_mutex". */
    static final Custom.Operations OPS = new CustomMutex();

    /** Size for "_mutex". */
    static final int SIZE = 1;

    /**
     * Only one instance of this class.
     */
    private CustomMutex() {
        super("_mutex", true, false, false);
    } // end empty constructor

    /**
     * {@inheritDoc}
     */
    @Override
    public int compare(final Value v1,
                       final Value v2,
                       final AtomicBoolean unordered) {
        return Custom.compareUsingMagicNumbers(v1, v2);
    } // end method 'compare(Value, Value, AtomicBoolean)'

} // end class 'CustomMutex'
