/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.FalseExit;
import fr.x9c.cadmium.kernel.Fatal;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;
import fr.x9c.cadmium.primitives.stdlib.Sys;

/**
 * This class provides implementation for 'unix_exit' primitive.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Exit {

    /**
     * No instance of this class.
     */
    private Exit() {
    } // end empty constructor

    /**
     * According to context, either exits the JVM or just stops the program.
     * @see fr.x9c.cadmium.primitives.stdlib.Sys#caml_sys_exit(CodeRunner, Value)
     * @param ctxt context
     * @param n return code of JVM
     * @return <i>unit</i> (never returns ...)
     * @throws FalseExit if false exit is activated for the code runner
     */
    @Primitive
    public static Value unix_exit(final CodeRunner ctxt,
                                  final Value n)
        throws Fail.Exception, Fatal.Exception, FalseExit {
        return Sys.caml_sys_exit(ctxt, n);
    } // end method 'unix_exit(CodeRunner, Value)'

} // end class 'Exit'
