/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import java.net.InetAddress;
import java.net.UnknownHostException;

import fr.x9c.cadmium.kernel.Block;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for 'unix_gethostname' primitive.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Gethostname {

    /**
     * No instance of this class.
     */
    private Gethostname() {
    } // end empty constructor

    /**
     * Returns the host name, as a string.
     * @param ctxt context
     * @param unit ignored
     * @return the host name, as a string
     * @throws Fail.Exception if it is impossible to get host name
     */
    @Primitive
    public static Value unix_gethostname(final CodeRunner ctxt,
                                         final Value unit)
        throws Fail.Exception {
        try {
            return Value.createFromBlock(Block.createString(InetAddress.getLocalHost().getHostName()));
        } catch (final UnknownHostException uhe) {
            Unix.fail(ctxt, "gethostname", uhe);
            return Value.UNIT; // never reached
        } // end try/catch
    } // end method 'unix_gethostname(CodeRunner, Value)'

} // end class 'Gethostname'
