/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for service-related primitives.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Getserv {

    /**
     * No instance of this class.
     */
    private Getserv() {
    } // end empty constructor

    /**
     * Raises <i>Not_found</i>.
     * @param ctxt context
     * @param name ignored
     * @param proto ignored
     * @return <i>unit</i>
     * @throws Fail.Exception always
     */
    @Primitive
    public static Value unix_getservbyname(final CodeRunner ctxt,
                                           final Value name,
                                           final Value proto)
        throws Fail.Exception {
        Fail.raiseNotFound();
        return Value.UNIT; // never reached
    } // end method 'unix_getservbyname(CodeRunner, Value, Value)'

    /**
     * Raises <i>Not_found</i>.
     * @param ctxt context
     * @param port ignored
     * @param proto ignored
     * @return <i>unit</i>
     * @throws Fail.Exception always
     */
    @Primitive
    public static Value unix_getservbyport(final CodeRunner ctxt,
                                           final Value port,
                                           final Value proto)
        throws Fail.Exception {
        Fail.raiseNotFound();
        return Value.UNIT; // never reached
    } // end method 'unix_getservbyport(CodeRunner, Value, Value)'

} // end class 'Getserv'
