/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import java.io.OutputStream;

import fr.x9c.cadmium.kernel.Channel;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for tty-related primitives.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Isatty {

    /**
     * No instance of this class.
     */
    private Isatty() {
    } // end empty constructor

    /**
     * Tests whether the passed file descriptor denotes a tty.
     * @param ctxt context
     * @param fd file descriptor to test
     * @return <i>true</i> if and only if the passed descriptor is equal to one
     * @throws Fail.Exception if the passed file descriptor is invalid
     */
    @Primitive
    public static Value unix_isatty(final CodeRunner ctxt,
                                    final Value fd)
        throws Fail.Exception {
        final Channel ch = ctxt.getContext().getChannel(fd.asLong());
        if (ch == null) {
            Unix.fail(ctxt, "iastty", Unix.INVALID_DESCRIPTOR_MSG);
            return Value.UNIT; // never reached
        } // end if
        final OutputStream os = ch.asOutputStream();
        return (os == System.out) || (os == System.err) ? Value.TRUE : Value.FALSE;
    } // end method 'unix_isatty(CodeRunner, Value)'

} // end class 'Isatty'
