/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import java.io.File;

import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for 'unix_rmdir' primitive.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Rmdir {

    /**
     * No instance of this class.
     */
    private Rmdir() {
    } // end empty constructor

    /**
     * Deletes a directory (that should be empty).
     * @param ctxt context
     * @param path directory to delete
     * @return <i>unit</i>
     * @throws Fail.Exception if directory does not exist
     * @throws Fail.Exception if directory cannot be deleted
     */
    @Primitive
    public static Value unix_rmdir(final CodeRunner ctxt,
                                   final Value path)
        throws Fail.Exception {
        final File f = ctxt.getContext().getRealFile(path);
        if (!f.delete()) {
            Unix.fail(ctxt, "rmdir", "cannot delete directory");
        } // end if
        return Value.UNIT;
    } // end method 'unix_rmdir(CodeRunner, Value)'

} // end class 'Rmdir'
