/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import java.net.InetAddress;
import java.net.UnknownHostException;

import fr.x9c.cadmium.kernel.Block;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Fail;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for 'unix_string_of_inet_addr' primitive.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Strofaddr {

    /**
     * No instance of this class.
     */
    private Strofaddr() {
    } // end empty constructor

    /**
     * Converts an address into a string with the following format:
     * <i>a.b.c.d</i>.
     * @param ctxt context
     * @param a address to convert
     * @return corresponding string
     * @throws Fail.Exception if host is unknown
     */
    @Primitive
    public static Value unix_string_of_inet_addr(final CodeRunner ctxt,
                                                 final Value a)
        throws Fail.Exception {
        try {
            final InetAddress addr = InetAddress.getByAddress(a.asBlock().getBytes());
            return Value.createFromBlock(Block.createString(addr.getHostAddress()));
        } catch (final UnknownHostException uhe) {
            Unix.fail(ctxt, "string_of_inet_addr", uhe);
            return Value.UNIT; // never reached
        } // end try/catch
    } // end method 'unix_string_of_inet_addr(CodeRunner, Value)'

} // end class 'Strofaddr'
