/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.primitives.unix;

import fr.x9c.cadmium.kernel.Block;
import fr.x9c.cadmium.kernel.CodeRunner;
import fr.x9c.cadmium.kernel.Primitive;
import fr.x9c.cadmium.kernel.PrimitiveProvider;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class provides implementation for 'unix_times' primitive.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
@PrimitiveProvider
public final class Times {

    /**
     * No instance of this class.
     */
    private Times() {
    } // end empty constructor

    /**
     * Returns a record of four floats representing execution time of the
     * process.
     * @param ctxt context
     * @param unit ignored
     * @return <i>(time_since_execution_start, 0.0, 0.0, 0.0)</i>
     */
    @Primitive
    public static Value unix_times(final CodeRunner ctxt,
                                   final Value unit) {
        final double time =
            ((double) (System.currentTimeMillis() - ctxt.getContext().getStart()))
            / Unix.MILLISECS_PER_SEC;
        final Block res = Block.createDoubleArray(4);
        res.setDouble(0, time);
        res.setDouble(1, 0.0);
        res.setDouble(2, 0.0);
        res.setDouble(3, 0.0);
        return Value.createFromBlock(res);
    } // end method 'unix_times(CodeRunner, Value)'

} // end class 'Times'
