/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.support.servlet;

import javax.servlet.ServletException;

import fr.x9c.cadmium.kernel.AbstractNativeRunner;
import fr.x9c.cadmium.kernel.Context;
import fr.x9c.cadmium.kernel.Value;

/**
 * This class is the parent class of all native servlets. <br/>
 * The servlet behaviour is defined by callbacks registered on the OCaml side
 * using <tt>Servlet.Servlet.register</tt>.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.1
 * @since 1.1
 */
public abstract class AbstractNativeServlet extends Servlet {

    /** Underlying native runner. */
    public static AbstractNativeRunner runner;

    /** Serialization UID. */
    static final long serialVersionUID = 6188764312212871057L;

    /**
     * Empty constructor.
     */
    public AbstractNativeServlet() {
        super();
    } // end empty constructor

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Value callback(final String cb, final Value... args)
        throws ServletException {
        assert cb != null : "null cb";
        try {
            final Value closure =
                AbstractNativeServlet.runner.getContext().getCallback(cb);
            return AbstractNativeServlet.runner.callback(closure, args);
        } catch (final Exception e) {
            throw new ServletException("Callback error", e);
        } // end try/catch
    } // end method 'callback(String, Value...)'

    /**
     * {@inheritDoc}
     */
    @Override
    protected final void callbackNoException(final String cb,
                                             final Value... args) {
        assert cb != null : "null cb";
        try {
            final Value closure =
                AbstractNativeServlet.runner.getContext().getCallback(cb);
            AbstractNativeServlet.runner.callback(closure, args);
        } catch (final Exception e) {
            // nothing to do
        } // end try/catch
    } // end method 'callback(String, Value...)'

    /**
     * {@inheritDoc}
     */
    @Override
    protected final boolean callbackOption(final String cb,
                                           final Value... args)
        throws ServletException {
        assert cb != null : "null cb";
        if (getCallback(cb) != null) {
            try {
                final Value closure =
                    AbstractNativeServlet.runner.getContext().getCallback(cb);
                AbstractNativeServlet.runner.callback(closure, args);
            } catch (final Exception e) {
                throw new ServletException("Callback error", e);
            } // end try/catch
            return false;
        } else {
            return true;
        } // end if/else
    } // end method 'callbackOption(String, Value...)'

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Value getCallback(final String cb) {
        return AbstractNativeServlet.runner.getContext().getCallback(cb);
    } // end method 'getCallback(String)'

    /**
     * {@inheritDoc}
     */
    @Override
    protected final Context getContext() {
        return AbstractNativeServlet.runner.getContext();
    } // end method 'getContext()'

} // end class 'AbstractNativeServlet'
