/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.support.values;

import fr.x9c.cadmium.kernel.Value;

/**
 * This class defines values of the <i>bool</i> type.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.0
 * @since 1.0
 */
public final class Bool implements ToValue {

    /** The actual value. */
    private final boolean value;

    /**
     * Constructs a boolean value.
     * @param v boolean value
     */
    public Bool(final boolean v) {
        this.value = v;
    } // end constructor(boolean)

    /**
     * Returns the value.
     * @return the value
     */
    public boolean get() {
        return this.value;
    } // end method 'get()'

    /**
     * {@inheritDoc}
     */
    @Override
    public Value toValue() {
        return this.value ? Value.TRUE : Value.FALSE;
    } // end method 'toValue()'

    /**
     * {@inheritDoc}
     */
    @Override
    public int hashCode() {
        return this.value ? 1 : 0;
    } // end method 'hashCode()'

    /**
     * {@inheritDoc}
     */
    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof Bool) {
            final Bool that = (Bool) obj;
            return this.value == that.value;
        } else {
            return false;
        } // end if/else
    } // end method 'equals(Object)'

    /**
     * {@inheritDoc}
     */
    @Override
    public String toString() {
        return Boolean.toString(this.value);
    } // end method 'toString()'

} // end class 'Bool'
