/*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package fr.x9c.cadmium.util;

import java.io.IOException;
import java.io.OutputStream;

/**
 * This class implements an output stream whose data destination may be
 * redirected, that is changed.
 *
 * @author <a href="mailto:cadmium@x9c.fr">Xavier Clerc</a>
 * @version 1.1
 * @since 1.1
 */
public final class RedirectedOutputStream extends OutputStream {

    /** Current data destination. */
    private OutputStream current;

    /**
     * Constructs a redirectable output stream from any output stream.
     * @param out data destination - should not be <tt>null</tt>
     */
    public RedirectedOutputStream(final OutputStream out) {
        assert out != null : "null out";
        this.current = out;
    } // end constructor(OutputStream)

    /**
     * Changes data destination.
     * @param out data destination - should not be <tt>null</tt>
     */
    public void redirect(final OutputStream out) {
        assert out != null : "null out";
        this.current = out;
    } // end method 'redirect(OutputStream)'

    /**
     * {@inheritDoc}
     */
    @Override
    public void write(final int b) throws IOException {
        this.current.write(b);
    } // end method 'write(int)'

} // end class 'RedirectedOutputStream'
