(*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

module Float =
  struct
    external get_e : unit -> float =
      "cadmiummath_float_get_e"

    external get_pi : unit -> float =
      "cadmiummath_float_get_pi"

    let e = get_e ()

    let pi = get_pi ()

    external abs : float -> float =
      "cadmiummath_float_abs"

    external acos : float -> float =
      "cadmiummath_float_acos"

    external asin : float -> float =
      "cadmiummath_float_asin"

    external atan : float -> float =
      "cadmiummath_float_atan"

    external atan2 : float -> float -> float =
      "cadmiummath_float_atan2"

    external cbrt : float -> float =
      "cadmiummath_float_cbrt"

    external ceil : float -> float =
      "cadmiummath_float_ceil"

    external copy_sign : float -> float -> float =
      "cadmiummath_float_copy_sign"

    external cos : float -> float =
      "cadmiummath_float_cos"

    external cosh : float -> float =
      "cadmiummath_float_cosh"

    external exp : float -> float =
      "cadmiummath_float_exp"

    external exp_m1 : float -> float =
      "cadmiummath_float_exp_m1"

    external floor : float -> float =
      "cadmiummath_float_floor"

    external get_exponent : float -> int =
      "cadmiummath_float_get_exponent"

    external hypot : float -> float -> float =
      "cadmiummath_float_hypot"

    external ieee_remainder : float -> float -> float =
      "cadmiummath_float_ieee_remainder"

    external log : float -> float =
      "cadmiummath_float_log"

    external log10 : float -> float =
      "cadmiummath_float_log10"

    external log1p : float -> float =
      "cadmiummath_float_log1p"

    external max : float -> float -> float =
      "cadmiummath_float_max"

    external min : float -> float -> float =
      "cadmiummath_float_min"

    external next_after : float -> float -> float =
      "cadmiummath_float_next_after"

    external next_up : float -> float =
      "cadmiummath_float_next_up"

    external pow : float -> float -> float =
      "cadmiummath_float_pow"

    external random : unit -> float =
      "cadmiummath_float_random"

    external rint : float -> float =
      "cadmiummath_float_rint"

    external round : float -> int64 =
      "cadmiummath_float_round"

    external scalb : float -> int -> float =
      "cadmiummath_float_scalb"

    external signum : float -> float =
      "cadmiummath_float_signum"

    external sin : float -> float =
      "cadmiummath_float_sin"

    external sinh : float -> float =
      "cadmiummath_float_sinh"

    external sqrt : float -> float =
      "cadmiummath_float_sqrt"

    external tan : float -> float =
      "cadmiummath_float_tan"

    external tanh : float -> float =
      "cadmiummath_float_tanh"

    external to_degrees : float -> float =
      "cadmiummath_float_to_degrees"

    external to_radians : float -> float =
      "cadmiummath_float_to_radians"

    external ulp : float -> float =
      "cadmiummath_float_ulp"
  end

module StrictFloat =
  struct
    external get_e : unit -> float =
      "cadmiummath_strictfloat_get_e"

    external get_pi : unit -> float =
      "cadmiummath_strictfloat_get_pi"

    let e = get_e ()

    let pi = get_pi ()

    external abs : float -> float =
      "cadmiummath_strictfloat_abs"

    external acos : float -> float =
      "cadmiummath_strictfloat_acos"

    external asin : float -> float =
      "cadmiummath_strictfloat_asin"

    external atan : float -> float =
      "cadmiummath_strictfloat_atan"

    external atan2 : float -> float -> float =
      "cadmiummath_strictfloat_atan2"

    external cbrt : float -> float =
      "cadmiummath_strictfloat_cbrt"

    external ceil : float -> float =
      "cadmiummath_strictfloat_ceil"

    external copy_sign : float -> float -> float =
      "cadmiummath_strictfloat_copy_sign"

    external cos : float -> float =
      "cadmiummath_strictfloat_cos"

    external cosh : float -> float =
      "cadmiummath_strictfloat_cosh"

    external exp : float -> float =
      "cadmiummath_strictfloat_exp"

    external exp_m1 : float -> float =
      "cadmiummath_strictfloat_exp_m1"

    external floor : float -> float =
      "cadmiummath_strictfloat_floor"

    external get_exponent : float -> int =
      "cadmiummath_strictfloat_get_exponent"

    external hypot : float -> float -> float =
      "cadmiummath_strictfloat_hypot"

    external ieee_remainder : float -> float -> float =
      "cadmiummath_strictfloat_ieee_remainder"

    external log : float -> float =
      "cadmiummath_strictfloat_log"

    external log10 : float -> float =
      "cadmiummath_strictfloat_log10"

    external log1p : float -> float =
      "cadmiummath_strictfloat_log1p"

    external max : float -> float -> float =
      "cadmiummath_strictfloat_max"

    external min : float -> float -> float =
      "cadmiummath_strictfloat_min"

    external next_after : float -> float -> float =
      "cadmiummath_strictfloat_next_after"

    external next_up : float -> float =
      "cadmiummath_strictfloat_next_up"

    external pow : float -> float -> float =
      "cadmiummath_strictfloat_pow"

    external random : unit -> float =
      "cadmiummath_strictfloat_random"

    external rint : float -> float =
      "cadmiummath_strictfloat_rint"

    external round : float -> int64 =
      "cadmiummath_strictfloat_round"

    external scalb : float -> int -> float =
      "cadmiummath_strictfloat_scalb"

    external signum : float -> float =
      "cadmiummath_strictfloat_signum"

    external sin : float -> float =
      "cadmiummath_strictfloat_sin"

    external sinh : float -> float =
      "cadmiummath_strictfloat_sinh"

    external sqrt : float -> float =
      "cadmiummath_strictfloat_sqrt"

    external tan : float -> float =
      "cadmiummath_strictfloat_tan"

    external tanh : float -> float =
      "cadmiummath_strictfloat_tanh"

    external to_degrees : float -> float =
      "cadmiummath_strictfloat_to_degrees"

    external to_radians : float -> float =
      "cadmiummath_strictfloat_to_radians"

    external ulp : float -> float =
      "cadmiummath_strictfloat_ulp"
  end

module Random =
  struct
    type t

    external make : unit -> t =
      "cadmiummath_random_make"

    external make_of_int64 : int64 -> t =
      "cadmiummath_random_make_of_int64"

    external next_boolean : t -> bool =
      "cadmiummath_random_next_boolean"

    external next_bytes : t -> string -> unit =
      "cadmiummath_random_next_bytes"

    external next_float : t -> float =
      "cadmiummath_random_next_float"

    external next_gaussian : t -> float =
      "cadmiummath_random_next_gaussian"

    external next_int : t -> int =
      "cadmiummath_random_next_int"

    external next_int_n : t -> int -> int =
      "cadmiummath_random_next_int_n"

    external next_int32 : t -> int32 =
      "cadmiummath_random_next_int32"

    external next_int32_n : t -> int32 -> int32 =
      "cadmiummath_random_next_int32_n"

    external next_int64 : t -> int64 =
      "cadmiummath_random_next_int64"

    external set_seed : t -> int64 -> unit =
      "cadmiummath_random_set_seed"
  end

module BigInteger =
  struct
    type t

    external get_constant : int -> t =
      "cadmiummath_biginteger_get_constant"

    let one = get_constant 1

    let ten = get_constant 10

    let zero = get_constant 0

    external make_of_bytes : string -> t =
      "cadmiummath_biginteger_make_of_bytes"

    external make_of_signum_bytes : int -> string -> t =
      "cadmiummath_biginteger_make_of_signum_bytes"

    external make_prime_of_random : int -> int -> Random.t -> t =
      "cadmiummath_biginteger_make_prime_of_random"

    external make_of_random : int -> Random.t -> t =
      "cadmiummath_biginteger_make_of_random"

    external make_of_string : string -> t =
      "cadmiummath_biginteger_make_of_string"

    external make_of_string_radix : string -> int -> t =
      "cadmiummath_biginteger_make_of_string_radix"

    external make_of_int64 : int64 -> t =
      "cadmiummath_biginteger_make_of_int64"

    external abs : t -> t =
      "cadmiummath_biginteger_abs"

    external add : t -> t -> t =
      "cadmiummath_biginteger_add"

    external and_ : t -> t -> t =
      "cadmiummath_biginteger_and_"

    external and_not : t -> t -> t =
      "cadmiummath_biginteger_and_not"

    external bit_count : t -> int =
      "cadmiummath_biginteger_bit_count"

    external bit_length : t -> int =
      "cadmiummath_biginteger_bit_length"

    external clear_bit : t -> int -> t =
      "cadmiummath_biginteger_clear_bit"

    external compare_to : t -> t -> int =
      "cadmiummath_biginteger_compare_to"

    external divide : t -> t -> t =
      "cadmiummath_biginteger_divide"

    external divide_and_remainder : t -> t -> (t * t) =
      "cadmiummath_biginteger_divide_and_remainder"

    external equals : t -> t -> bool =
      "cadmiummath_biginteger_equals"

    external flip_bit : t -> int -> t =
      "cadmiummath_biginteger_flip_bit"

    external float_value : t -> float =
      "cadmiummath_biginteger_float_value"

    external gcd : t -> t -> t =
      "cadmiummath_biginteger_gcd"

    external get_lowest_set_bit : t -> int =
      "cadmiummath_biginteger_get_lowest_set_bit"

    external hash_code : t -> int32 =
      "cadmiummath_biginteger_hash_code"

    external int_value : t -> int =
      "cadmiummath_biginteger_int_value"

    external int32_value : t -> int32 =
      "cadmiummath_biginteger_int32_value"

    external int64_value : t -> int64 =
      "cadmiummath_biginteger_int64_value"

    external is_probably_prime : t -> int -> bool =
      "cadmiummath_biginteger_is_probably_prime"

    external max : t -> t -> t =
      "cadmiummath_biginteger_max"

    external min : t -> t -> t =
      "cadmiummath_biginteger_min"

    external mod_ : t -> t -> t =
      "cadmiummath_biginteger_mod_"

    external mod_inverse : t -> t -> t =
      "cadmiummath_biginteger_mod_inverse"

    external mod_pow : t -> t -> t -> t =
      "cadmiummath_biginteger_mod_pow"

    external multiply : t -> t -> t =
      "cadmiummath_biginteger_multiply"

    external negate : t -> t =
      "cadmiummath_biginteger_negate"

    external next_probable_prime : t -> t =
      "cadmiummath_biginteger_next_probable_prime"

    external not : t -> t =
      "cadmiummath_biginteger_not"

    external or_ : t -> t -> t =
      "cadmiummath_biginteger_or_"

    external pow : t -> int -> t =
      "cadmiummath_biginteger_pow"

    external remainder : t -> t -> t =
      "cadmiummath_biginteger_remainder"

    external set_bit : t -> int -> t =
      "cadmiummath_biginteger_set_bit"

    external shift_left : t -> int -> t =
      "cadmiummath_biginteger_shift_left"

    external shift_right : t -> int -> t =
      "cadmiummath_biginteger_shift_right"

    external signum : t -> int =
      "cadmiummath_biginteger_signum"

    external subtract : t -> t -> t =
      "cadmiummath_biginteger_subtract"

    external test_bit : t -> int -> bool =
      "cadmiummath_biginteger_test_bit"

    external to_bytes : t -> string =
      "cadmiummath_biginteger_to_bytes"

    external to_string : t -> string =
      "cadmiummath_biginteger_to_string"

    external to_string_radix : t -> int -> string =
      "cadmiummath_biginteger_to_string_radix"

    external xor : t -> t -> t =
      "cadmiummath_biginteger_xor"
  end

module BigDecimal =
  struct
    type rounding_mode =
      | CEILING
      | DOWN
      | FLOOR
      | HALF_DOWN
      | HALF_EVEN
      | HALF_UP
      | UNNECESSARY
      | UP

    type math_context =
      | Decimal32
      | Decimal64
      | Decimal128
      | Unlimited
      | Custom of int * rounding_mode

    type t

    external get_constant : int -> t =
      "cadmiummath_bigdecimal_get_constant"

    let one = get_constant 1

    let ten = get_constant 10

    let zero = get_constant 0

    external make_of_big_integer : BigInteger.t -> t =
      "cadmiummath_bigdecimal_make_of_big_integer"

    external make_of_big_integer_ctxt : math_context -> BigInteger.t -> t =
      "cadmiummath_bigdecimal_make_of_big_integer_ctxt"

    external make_of_big_integer_scale : BigInteger.t -> int -> t =
      "cadmiummath_bigdecimal_make_of_big_integer_scale"

    external make_of_big_integer_scale_ctxt : math_context -> BigInteger.t -> int -> t =
      "cadmiummath_bigdecimal_make_of_big_integer_scale_ctxt"

    external make_of_float : float -> t =
      "cadmiummath_bigdecimal_make_of_float"

    external make_of_float_ctxt : math_context -> float -> t =
      "cadmiummath_bigdecimal_make_of_float_ctxt"

    external make_of_int32 : int32 -> t =
      "cadmiummath_bigdecimal_make_of_int32"

    external make_of_int32_ctxt : math_context -> int32 -> t =
      "cadmiummath_bigdecimal_make_of_int32_ctxt"

    external make_of_int64 : int64 -> t =
      "cadmiummath_bigdecimal_make_of_int64"

    external make_of_int64_ctxt : math_context -> int64 -> t =
      "cadmiummath_bigdecimal_make_of_int64_ctxt"

    external make_of_string : string -> t =
      "cadmiummath_bigdecimal_make_of_string"

    external make_of_string_ctxt : math_context -> string -> t =
      "cadmiummath_bigdecimal_make_of_string_ctxt"

    external abs : t -> t =
      "cadmiummath_bigdecimal_abs"

    external abs_ctxt : math_context -> t -> t =
      "cadmiummath_bigdecimal_abs_ctxt"

    external add : t -> t -> t =
      "cadmiummath_bigdecimal_add"

    external add_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_add_ctxt"

    external byte_value_exact : t -> int =
      "cadmiummath_bigdecimal_byte_value_exact"

    external compare_to : t -> t -> int =
      "cadmiummath_bigdecimal_compare_to"

    external divide : t -> t -> t =
      "cadmiummath_bigdecimal_divide"

    external divide_scale_mode : int -> rounding_mode -> t -> t -> t =
      "cadmiummath_bigdecimal_divide_scale_mode"

    external divide_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_divide_ctxt"

    external divide_mode : rounding_mode -> t -> t -> t =
      "cadmiummath_bigdecimal_divide_mode"

    external divide_and_remainder : t -> t -> (t * t) =
      "cadmiummath_bigdecimal_divide_and_remainder"

    external divide_and_remainder_ctxt : math_context -> t -> t -> (t * t) =
      "cadmiummath_bigdecimal_divide_and_remainder_ctxt"

    external divide_to_integral_value : t -> t -> t =
      "cadmiummath_bigdecimal_divide_to_integral_value"

    external divide_to_integral_value_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_divide_to_integral_value_ctxt"

    external equals : t -> t -> bool =
      "cadmiummath_bigdecimal_equals"

    external float_value : t -> float =
      "cadmiummath_bigdecimal_float_value"

    external hash_code : t -> int32 =
      "cadmiummath_bigdecimal_hash_code"

    external int_value : t -> int =
      "cadmiummath_bigdecimal_int_value"

    external int_value_exact : t -> int =
      "cadmiummath_bigdecimal_int_value_exact"

    external int32_value : t -> int32 =
      "cadmiummath_bigdecimal_int32_value"

    external int32_value_exact : t -> int32 =
      "cadmiummath_bigdecimal_int32_value_exact"

    external int64_value : t -> int64 =
      "cadmiummath_bigdecimal_int64_value"

    external int64_value_exact : t -> int64 =
      "cadmiummath_bigdecimal_int64_value_exact"

    external max : t -> t -> t =
      "cadmiummath_bigdecimal_max"

    external min : t -> t -> t =
      "cadmiummath_bigdecimal_min"

    external move_point_left : t -> int -> t =
      "cadmiummath_bigdecimal_move_point_left"

    external move_point_right : t -> int -> t =
      "cadmiummath_bigdecimal_move_point_right"

    external multiply : t -> t -> t =
      "cadmiummath_bigdecimal_multiply"

    external multiply_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_multiply_ctxt"

    external negate : t -> t =
      "cadmiummath_bigdecimal_negate"

    external negate_ctxt : math_context -> t -> t =
      "cadmiummath_bigdecimal_negate_ctxt"

    external plus : t -> t =
      "cadmiummath_bigdecimal_plus"

    external plus_ctxt : math_context -> t -> t =
      "cadmiummath_bigdecimal_plus_ctxt"

    external pow : t -> int -> t =
      "cadmiummath_bigdecimal_pow"

    external pow_ctxt : math_context -> t -> int -> t =
      "cadmiummath_bigdecimal_pow_ctxt"

    external precision : t -> int =
      "cadmiummath_bigdecimal_precision"

    external remainder : t -> t -> t =
      "cadmiummath_bigdecimal_remainder"

    external remainder_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_remainder_ctxt"

    external round : math_context -> t -> t =
      "cadmiummath_bigdecimal_round"

    external scale : t -> int =
      "cadmiummath_bigdecimal_scale"

    external scale_by_power_of_ten : t -> int -> t =
      "cadmiummath_bigdecimal_scale_by_power_of_ten"

    external set_scale : t -> int -> t =
      "cadmiummath_bigdecimal_set_scale"

    external set_scale_mode : rounding_mode -> t -> int -> t =
      "cadmiummath_bigdecimal_set_scale_mode"

    external short_value_exact : t -> int =
      "cadmiummath_bigdecimal_short_value_exact"

    external signum : t -> int =
      "cadmiummath_bigdecimal_signum"

    external strip_trailing_zeros : t -> t =
      "cadmiummath_bigdecimal_strip_trailing_zeros"

    external subtract : t -> t -> t =
      "cadmiummath_bigdecimal_subtract"

    external subtract_ctxt : math_context -> t -> t -> t =
      "cadmiummath_bigdecimal_subtract_ctxt"

    external to_big_integer : t -> BigInteger.t =
      "cadmiummath_bigdecimal_to_big_integer"

    external to_big_integer_exact : t -> BigInteger.t =
      "cadmiummath_bigdecimal_to_big_integer_exact"

    external to_engineering_string : t -> string =
      "cadmiummath_bigdecimal_to_engineering_string"

    external to_plain_string : t -> string =
      "cadmiummath_bigdecimal_to_plain_string"

    external to_string : t -> string =
      "cadmiummath_bigdecimal_to_string"

    external ulp : t -> t =
      "cadmiummath_bigdecimal_ulp"

    external unscaled_value : t -> BigInteger.t =
      "cadmiummath_bigdecimal_unscaled_value"
  end

let () = (* to ensure that the Cadmium library is initialized *)
  ignore (Cadmium.is_available ())
