(*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

module Cookie =
  struct
    type t

    external make : string -> string -> t =
      "cadmiumservlet_cookie_make"

    external clone : t -> t =
      "cadmiumservlet_cookie_clone"

    external get_comment : t -> string =
      "cadmiumservlet_cookie_get_comment"

    external get_domain : t -> string =
      "cadmiumservlet_cookie_get_domain"

    external get_max_age : t -> int32 =
      "cadmiumservlet_cookie_get_max_age"

    external get_name : t -> string =
      "cadmiumservlet_cookie_get_name"

    external get_path : t -> string =
      "cadmiumservlet_cookie_get_path"

    external get_secure : t -> bool =
      "cadmiumservlet_cookie_get_secure"

    external get_value : t -> string =
      "cadmiumservlet_cookie_get_value"

    external get_version : t -> int32 =
      "cadmiumservlet_cookie_get_version"

    external set_comment : t -> string -> unit =
      "cadmiumservlet_cookie_set_comment"

    external set_domain : t -> string -> unit =
      "cadmiumservlet_cookie_set_domain"

    external set_max_age : t -> int32 -> unit =
      "cadmiumservlet_cookie_set_max_age"

    external set_path : t -> string -> unit =
      "cadmiumservlet_cookie_set_path"

    external set_secure : t -> bool -> unit =
      "cadmiumservlet_cookie_set_secure"

    external set_value : t -> string -> unit =
      "cadmiumservlet_cookie_set_value"

    external set_version : t -> int32 -> unit =
      "cadmiumservlet_cookie_set_version"
  end


module Context =
  struct
    type t

    external get_attribute : t -> string -> Cadmium.java_object =
      "cadmiumservlet_context_get_attribute"

    let get_attribute_obj x s =
      new CadmiumObj.jObject (`Cd'init (get_attribute x s))

    external get_attribute_names : t -> string list =
      "cadmiumservlet_context_get_attribute_names"

    external get_context : t -> string -> t =
      "cadmiumservlet_context_get_context"

    external get_context_path : t -> string =
      "cadmiumservlet_context_get_context_path"

    external get_init_parameter : t -> string -> string =
      "cadmiumservlet_context_get_init_parameter"

    external get_init_parameter_names : t -> string list =
      "cadmiumservlet_context_get_init_parameter_names"

    external get_major_version : t -> int32 =
      "cadmiumservlet_context_get_major_version"

    external get_mime_type : t -> string -> string =
      "cadmiumservlet_context_get_mime_type"

    external get_minor_version : t -> int32 =
      "cadmiumservlet_context_get_minor_version"

    external get_real_path : t -> string -> string =
      "cadmiumservlet_context_get_real_path"

    external get_resource_as_url : t -> string -> string =
      "cadmiumservlet_context_get_resource_as_url"

    external get_resource_as_stream : t -> string -> in_channel =
      "cadmiumservlet_context_get_resource_as_stream"

    external get_resource_paths : t -> string -> string list =
      "cadmiumservlet_context_get_resource_paths"

    external get_server_info : t -> string =
      "cadmiumservlet_context_get_server_info"

    external get_servlet_context_name : t -> string =
      "cadmiumservlet_context_get_servlet_context_name"

    external log : t -> string -> unit =
      "cadmiumservlet_context_log"

    external log_exn : t -> string -> Cadmium.java_object -> unit =
      "cadmiumservlet_context_log_exn"

    let log_exn_obj x s o = log_exn x s o#cd'this

    external remove_attribute : t -> string -> unit =
      "cadmiumservlet_context_remove_attribute"

    external set_attribute : t -> string -> Cadmium.java_object -> unit =
      "cadmiumservlet_context_set_attribute"

    let set_attribute_obj x s o = set_attribute x s o#cd'this
  end


module Session =
  struct
    type t

    external get_attribute : t -> string -> Cadmium.java_object =
      "cadmiumservlet_session_get_attribute"

    let get_attribute_obj x s =
      new CadmiumObj.jObject (`Cd'init (get_attribute x s))

    external get_attribute_names : t -> string list =
      "cadmiumservlet_session_get_attribute_names"

    external get_creation_time : t -> int64 =
      "cadmiumservlet_session_get_creation_time"

    external get_id : t -> string =
      "cadmiumservlet_session_get_id"

    external get_last_accessed_time : t -> int64 =
      "cadmiumservlet_session_get_last_accessed_time"

    external get_max_inactive_interval : t -> int32 =
      "cadmiumservlet_session_get_max_inactive_interval"

    external get_servlet_context : t -> Context.t =
      "cadmiumservlet_session_get_servlet_context"

    external invalidate : t -> unit =
      "cadmiumservlet_session_invalidate"

    external is_new : t -> bool =
      "cadmiumservlet_session_is_new"

    external remove_attribute : t -> string -> unit =
      "cadmiumservlet_session_remove_attribute"

    external set_attribute : t -> string -> Cadmium.java_object -> unit =
      "cadmiumservlet_session_set_attribute"

    let set_attribute_obj x s o = set_attribute x s o#cd'this

    external set_max_inactive_interval : t -> int32 -> unit =
      "cadmiumservlet_session_set_max_inactive_interval"
  end


module Listeners =
  struct
    type session_activation_listener = {
        session_did_activate : Session.t -> unit;
        session_will_passivate : Session.t -> unit;
      }

    let register_session_activation_listener l =
      Callback.register
        "Servlet.session_activation_listener.session_did_activate"
        l.session_did_activate;
      Callback.register
        "Servlet.session_activation_listener.session_will_passivate"
        l.session_will_passivate;

    type session_attribute_listener = {
        attribute_added : Session.t -> string -> Cadmium.java_object -> unit;
        attribute_removed : Session.t -> string -> Cadmium.java_object -> unit;
        attribute_replaced : Session.t -> string -> Cadmium.java_object -> unit;
      }

    let register_session_attribute_listener l =
      Callback.register
        "Servlet.session_attribute_listener.attribute_added"
        l.attribute_added;
      Callback.register
        "Servlet.session_attribute_listener.attribute_removed"
        l.attribute_removed;
      Callback.register
        "Servlet.session_attribute_listener.attribute_replaced"
        l.attribute_replaced

    type session_binding_listener = {
        value_bound : Session.t -> string -> Cadmium.java_object -> unit;
        value_unbound : Session.t -> string -> Cadmium.java_object -> unit;
      }

    let register_session_binding_listener l =
      Callback.register
        "Servlet.session_binding_listener.value_bound"
        l.value_bound;
      Callback.register
        "Servlet.session_binding_listener.value_unbound"
        l.value_unbound

    type session_listener = {
        session_created : Session.t -> unit;
        session_destroyed : Session.t -> unit;
      }

    let register_session_listener l =
      Callback.register
        "Servlet.session_listener.session_created"
        l.session_created;
      Callback.register
        "Servlet.session_listener.session_destroyed"
        l.session_destroyed
  end


module Request =
  struct
    type t

    external get_attribute : t -> string -> Cadmium.java_object =
      "cadmiumservlet_request_get_attribute"

    let get_attribute_obj x s =
      new CadmiumObj.jObject (`Cd'init (get_attribute x s))

    external get_attribute_names : t -> string list =
      "cadmiumservlet_request_get_attribute_names"

    external get_auth_type : t -> string =
      "cadmiumservlet_request_get_auth_type"

    external get_character_encoding : t -> string =
      "cadmiumservlet_request_get_character_encoding"

    external get_content_length : t -> int32 =
      "cadmiumservlet_request_get_content_length"

    external get_content_type : t -> string =
      "cadmiumservlet_request_get_content_type"

    external get_context_path : t -> string =
      "cadmiumservlet_request_get_context_path"

    external get_cookies : t -> Cookie.t list =
      "cadmiumservlet_request_get_cookies"

    external get_date_header : t -> string -> int64 =
      "cadmiumservlet_request_get_date_header"

    external get_header : t -> string -> string =
      "cadmiumservlet_request_get_header"

    external get_header_names : t -> string list =
      "cadmiumservlet_request_get_header_names"

    external get_headers : t -> string -> string list =
      "cadmiumservlet_request_get_headers"

    external get_input_stream : t -> in_channel =
      "cadmiumservlet_request_get_input_stream"

    external get_int_header : t -> string -> int32 =
      "cadmiumservlet_request_get_int_header"

    external get_local_addr : t -> string =
      "cadmiumservlet_request_get_local_addr"

    external get_locale : t -> string =
      "cadmiumservlet_request_get_locale"

    external get_locales : t -> string list =
      "cadmiumservlet_request_get_locales"

    external get_local_name : t -> string =
      "cadmiumservlet_request_get_local_name"

    external get_local_port : t -> int32 =
      "cadmiumservlet_request_get_local_port"

    external get_method : t -> string =
      "cadmiumservlet_request_get_method"

    external get_parameter : t -> string -> string =
      "cadmiumservlet_request_get_parameter"

    external get_parameter_list : t -> ((string * (string list)) list) =
      "cadmiumservlet_request_get_parameter_list"

    external get_parameter_names : t -> string list =
      "cadmiumservlet_request_get_parameter_names"

    external get_parameter_values : t -> string -> string list =
      "cadmiumservlet_request_get_parameter_values"

    external get_path_info : t -> string =
      "cadmiumservlet_request_get_path_info"

    external get_path_translated : t -> string =
      "cadmiumservlet_request_get_path_translated"

    external get_protocol : t -> string =
      "cadmiumservlet_request_get_protocol"

    external get_query_string : t -> string =
      "cadmiumservlet_request_get_query_string"

    external get_remote_addr : t -> string =
      "cadmiumservlet_request_get_remote_addr"

    external get_remote_host : t -> string =
      "cadmiumservlet_request_get_remote_host"

    external get_remote_port : t -> int32 =
      "cadmiumservlet_request_get_remote_port"

    external get_remote_user : t -> string =
      "cadmiumservlet_request_get_remote_user"

    external get_requested_session_id : t -> string =
      "cadmiumservlet_request_get_requested_session_id"

    external get_request_uri : t -> string =
      "cadmiumservlet_request_get_request_uri"

    external get_request_url : t -> string =
      "cadmiumservlet_request_get_request_url"

    external get_scheme : t -> string =
      "cadmiumservlet_request_get_scheme"

    external get_server_name : t -> string =
      "cadmiumservlet_request_get_server_name"

    external get_server_port : t -> int32 =
      "cadmiumservlet_request_get_server_port"

    external get_servlet_path : t -> string =
      "cadmiumservlet_request_get_servlet_path"

    external get_session : t -> Session.t =
      "cadmiumservlet_request_get_session"

    external get_session_creating : t -> bool -> Session.t =
      "cadmiumservlet_request_get_session_creating"

    external is_requested_session_id_from_cookie : t -> bool =
      "cadmiumservlet_request_is_requested_session_id_from_cookie"

    external is_requested_session_id_from_url : t -> bool =
      "cadmiumservlet_request_is_requested_session_id_from_url"

    external is_requested_session_id_valid : t -> bool =
      "cadmiumservlet_request_is_requested_session_id_valid"

    external is_secure : t -> bool =
      "cadmiumservlet_request_is_secure"

    external is_user_in_role : t -> string -> bool =
      "cadmiumservlet_request_is_user_in_role"

    external remove_attribute : t -> string -> unit =
      "cadmiumservlet_request_remove_attribute"

    external set_attribute : t -> string -> Cadmium.java_object -> unit =
      "cadmiumservlet_request_set_attribute"

    let set_attribute_obj x s o = set_attribute x s o#cd'this

    external set_character_encoding : t -> string -> unit =
      "cadmiumservlet_request_set_character_encoding"
  end


module Response =
  struct
    let sc_accepted = 202l

    let sc_bad_gateway = 502l

    let sc_bad_request = 400l

    let sc_conflict = 409l

    let sc_continue = 100l

    let sc_created = 201l

    let sc_expectation_failed = 417l

    let sc_forbidden = 403l

    let sc_found = 302l

    let sc_gateway_tiemout = 504l

    let sc_gone = 410l

    let sc_http_version_not_supported = 505l

    let sc_internal_server_error = 500l

    let sc_length_required = 411l

    let sc_method_not_allowed = 405l

    let sc_moved_permanently = 301l

    let sc_moved_temporarily = 302l

    let sc_multiple_choices = 300l

    let sc_no_content = 204l

    let sc_non_authoritative_information = 203l

    let sc_not_acceptable = 406l

    let sc_not_found = 404l

    let sc_not_implemented = 501l

    let sc_not_modified = 304l

    let sc_ok = 200l

    let sc_partial_content = 206l

    let sc_payment_required = 402l

    let sc_precondition_failed = 412l

    let sc_proxy_authentication_required = 407l

    let sc_request_entity_too_large = 413l

    let sc_request_timeout = 408l

    let sc_request_uri_too_long = 414l

    let sc_requested_range_not_satisfiable = 416l

    let sc_reset_content = 205l

    let sc_see_other = 303l

    let sc_service_unavailable = 503l

    let sc_switching_protocols = 101l

    let sc_temporary_redirect = 307l

    let sc_unauthorized = 401l

    let sc_unsupported_media_type = 415l

    let sc_use_proxy = 305l

    type t

    external add_cookie : t -> Cookie.t -> unit =
      "cadmiumservlet_response_add_cookie"

    external add_date_header : t -> string -> int64 -> unit =
      "cadmiumservlet_response_add_date_header"

    external add_header : t -> string -> string -> unit =
      "cadmiumservlet_response_add_header"

    external add_int_header : t -> string -> int32 -> unit =
      "cadmiumservlet_response_add_int_header"

    external contains_header : t -> string -> bool =
      "cadmiumservlet_response_contains_header"

    external encode_redirect_url : t -> string -> string =
      "cadmiumservlet_response_encode_redirect_url"

    external encode_url : t -> string -> string =
      "cadmiumservlet_response_encode_url"

    external flush_buffer : t -> unit =
      "cadmiumservlet_response_flush_buffer"

    external get_buffer_size : t -> int32 =
      "cadmiumservlet_response_get_buffer_size"

    external get_character_encoding : t -> string =
      "cadmiumservlet_response_get_character_encoding"

    external get_content_type : t -> string =
      "cadmiumservlet_response_get_content_type"

    external get_locale : t -> string =
      "cadmiumservlet_response_get_locale"

    external get_output_stream : t -> out_channel =
      "cadmiumservlet_response_get_output_stream"

    external is_committed : t -> bool =
      "cadmiumservlet_response_is_committed"

    external reset : t -> unit =
      "cadmiumservlet_response_reset"

    external reset_buffer : t -> unit =
      "cadmiumservlet_response_reset_buffer"

    external send_error : t -> int32 -> unit =
      "cadmiumservlet_response_send_error"

    external send_error_msg : t -> int32 -> string -> unit =
      "cadmiumservlet_response_send_error_msg"

    external send_redirect : t -> string -> unit =
      "cadmiumservlet_response_send_redirect"

    external set_buffer_size : t -> int32 -> unit =
      "cadmiumservlet_response_set_buffer_size"

    external set_character_encoding : t -> string -> unit =
      "cadmiumservlet_response_set_character_encoding"

    external set_content_length : t -> int32 -> unit =
      "cadmiumservlet_response_set_content_length"

    external set_content_type : t -> string -> unit =
      "cadmiumservlet_response_set_content_type"

    external set_date_header : t -> string -> int64 -> unit =
      "cadmiumservlet_response_set_date_header"

    external set_header : t -> string -> string -> unit =
      "cadmiumservlet_response_set_header"

    external set_int_header : t -> string -> int32 -> unit =
      "cadmiumservlet_response_set_int_header"

    external set_locale : t -> string -> unit =
      "cadmiumservlet_response_set_locale"

    external set_status : t -> int32 -> unit =
      "cadmiumservlet_response_set_status"
  end


module Servlet =
  struct
    type t

    external get_init_parameter : t -> string -> string =
      "cadmiumservlet_servlet_get_init_parameter"

    external get_init_parameter_names : t -> string list =
      "cadmiumservlet_servlet_get_init_parameter_names"

    external get_servlet_context : t -> Context.t =
      "cadmiumservlet_servlet_get_servlet_context"

    external get_servlet_name : t -> string =
      "cadmiumservlet_servlet_get_servlet_name"

    external log : t -> string -> unit =
      "cadmiumservlet_servlet_log"

    external log_exn : t -> string -> Cadmium.java_object -> unit =
      "cadmiumservlet_servlet_log_exn"

    let log_exn_obj x s o = log_exn x s o#cd'this

    type handler = {
        destroy : t -> unit;
        init : t -> unit;
        info : string;
        do_delete : (t -> Request.t -> Response.t -> unit) option;
        do_get : (t -> Request.t -> Response.t -> unit) option;
        do_head : (t -> Request.t -> Response.t -> unit) option;
        do_options : (t -> Request.t -> Response.t -> unit) option;
        do_post : (t -> Request.t -> Response.t -> unit) option;
        do_put : (t -> Request.t -> Response.t -> unit) option;
        do_trace : (t -> Request.t -> Response.t -> unit) option;
        get_last_modified : (t -> Request.t -> int64) option;
      }

    let register h =
      let register_if n = function
        | Some f -> Callback.register n f
        | None -> () in
      Callback.register "Servlet.destroy" h.destroy;
      Callback.register "Servlet.init" h.init;
      Callback.register "Servlet.info" h.info;
      register_if "Servlet.do_delete" h.do_delete;
      register_if "Servlet.do_get" h.do_get;
      register_if "Servlet.do_head" h.do_head;
      register_if "Servlet.do_options" h.do_options;
      register_if "Servlet.do_post" h.do_post;
      register_if "Servlet.do_put" h.do_put;
      register_if "Servlet.do_trace" h.do_trace;
      register_if "Servlet.get_last_modified" h.get_last_modified
  end


let () = (* to ensure that the Cadmium library is initialized *)
  ignore (Cadmium.is_available ())
