(*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

open CadmiumObj


(* Configuration *)

let is_debug_mode = CadmiumSwiXml.is_debug_mode

let is_macosx = CadmiumSwiXml.is_macosx

let is_macosx_supported = CadmiumSwiXml.is_macosx_supported

let set_macosx_support = CadmiumSwiXml.set_macosx_support

let set_default_locale o = CadmiumSwiXml.set_default_locale o#cd'this

let set_resource_bundle_name = CadmiumSwiXml.set_resource_bundle_name

let set_app_frame o = CadmiumSwiXml.set_app_frame o#cd'this

let get_app_frame () = new jObject (`Cd'init (CadmiumSwiXml.get_app_frame ()))


(* Engine object *)

class engine p =
  let this = match p with
  | `Void -> CadmiumSwiXml.java_object_of_engine (CadmiumSwiXml.make_engine ())
  | `Resource s -> CadmiumSwiXml.java_object_of_engine(CadmiumSwiXml.make_engine_from_resource s)
  | `Client (s : jObject) -> CadmiumSwiXml.java_object_of_engine (CadmiumSwiXml.make_engine_from_client s#cd'this)
  | `Cd'init t -> t in
  let that = Obj.magic this in
  object
    inherit (jObject (`Cd'init this))
    method cleanup = CadmiumSwiXml.cleanup that
    method clearIdMap = CadmiumSwiXml.clear_id_map that
    method find id = new jObject (`Cd'init (CadmiumSwiXml.find that id))
    method forget id = CadmiumSwiXml.forget that id
    method getAllComponents =
      let l = CadmiumSwiXml.get_all_components that in
      List.map (fun x -> new jObject (`Cd'init x)) l
    method getClassloader = new jObject (`Cd'init (CadmiumSwiXml.get_classloader that))
    method getClient = new jObject (`Cd'init (CadmiumSwiXml.get_client that))
    method getDescendants (obj : jObject) =
      let l = CadmiumSwiXml.get_descendants that obj#cd'this in
      List.map (fun x -> new jObject (`Cd'init x)) l
    method getIdMap =
      let l = CadmiumSwiXml.get_id_map that in
      List.map (fun (x, y) -> (x, new jObject (`Cd'init y))) l
    method getRootComponent = new jObject (`Cd'init (CadmiumSwiXml.get_root_component that))
    method insertFromChannel ch (c : jObject) = CadmiumSwiXml.insert_from_channel that ch c#cd'this
    method insertFromResource r (c : jObject) = CadmiumSwiXml.insert_from_resource that r c#cd'this
    method insertFromURL u (c : jObject) = CadmiumSwiXml.insert_from_url that u c#cd'this
    method renderFromChannel ch = new jObject (`Cd'init (CadmiumSwiXml.render_from_channel that ch))
    method renderFromFile f = new jObject (`Cd'init (CadmiumSwiXml.render_from_file that f))
    method renderFromResource r = new jObject (`Cd'init (CadmiumSwiXml.render_from_resource that r))
    method renderFromURL u = new jObject (`Cd'init (CadmiumSwiXml.render_from_url that u))
    method setActionListener (cmp : jObject) (lst : jObject) = CadmiumSwiXml.set_action_listener that cmp#cd'this lst#cd'this
    method setActionListenerByName id (lst : jObject) = CadmiumSwiXml.set_action_listener_by_name that id lst#cd'this
    method setClassloader (cl : jObject) = CadmiumSwiXml.set_classloader that cl#cd'this
    method setLocale (l : jObject) = CadmiumSwiXml.set_locale that l#cd'this
    method setResourceBundle rb = CadmiumSwiXml.set_resource_bundle that rb
  end
