(*
 * This file is part of Cadmium.
 * Copyright (C) 2007-2010 Xavier Clerc.
 *
 * Cadmium is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * Cadmium is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *)

(** Access to SwiXml engine (http://www.swixml.org). *)

(** This module is a straightforward translation of {i org.swixml.SwingEngine}.
   Each of the functions and methods may raise a [Cadmium.Java_exception] if
   the underlying Java method throws an exception. *)


(** {6 Configuration} *)

val is_debug_mode : unit -> bool
(** Tests whether SwiXml is in debug or release mode. *)

val is_macosx : unit -> bool
(** Tests whether the underlying platform is MacOS X. *)

val is_macosx_supported : unit -> bool
(** Tests whether MacOS X support is enabled. *)

val set_macosx_support : bool -> unit
(** Enables or disables MacOS X support. *)

val set_default_locale : CadmiumObj.jObject -> unit
(** Sets the default locale. The parameter should be a {i java.util.Locale}
   instance. *)

val set_resource_bundle_name : string -> unit
(** Sets the default resource bundle name. *)

val set_app_frame : CadmiumObj.jObject -> unit
(** Sets the global application frame, used as the parent for child dialogs.
   The parameter should be a {i java.awt.Frame} instance. *)

val get_app_frame : unit -> CadmiumObj.jObject
(** Returns the global application frame. *)


(** {6 Engine object} *)

(** One should refer to either [SwiXml] module or SwiXml javadoc for
   method documentation. *)

class engine :
  [< `Void | `Resource of string | `Client of CadmiumObj.jObject | `Cd'init of Cadmium.java_object ] ->
  object
    method cd'cast : CadmiumObj.jClass -> CadmiumObj.jObject
    method cd'equal : CadmiumObj.jObject -> bool
    method cd'instanceof : CadmiumObj.jClass -> bool
    method cd'isnull : bool
    method cd'notequal : CadmiumObj.jObject -> bool
    method cd'this : Cadmium.java_object
    method cleanup : int
    method clearIdMap : unit
    method clone : CadmiumObj.jObject
    method equals : CadmiumObj.jObject -> bool
    method find : string -> CadmiumObj.jObject
    method forget : string -> unit
    method getAllComponents : CadmiumObj.jObject list
    method getClass : CadmiumObj.jClass
    method getClassloader : CadmiumObj.jObject
    method getClient : CadmiumObj.jObject
    method getDescendants : CadmiumObj.jObject -> CadmiumObj.jObject list
    method getIdMap : (string * CadmiumObj.jObject) list
    method getRootComponent : CadmiumObj.jObject
    method hashCode : int32
    method insertFromChannel : in_channel -> CadmiumObj.jObject -> unit
    method insertFromResource : string -> CadmiumObj.jObject -> unit
    method insertFromURL : string -> CadmiumObj.jObject -> unit
    method notify : unit
    method notifyAll : unit
    method renderFromChannel : in_channel -> CadmiumObj.jObject
    method renderFromFile : string -> CadmiumObj.jObject
    method renderFromResource : string -> CadmiumObj.jObject
    method renderFromURL : string -> CadmiumObj.jObject
    method setActionListener : CadmiumObj.jObject -> CadmiumObj.jObject -> bool
    method setActionListenerByName : string -> CadmiumObj.jObject -> bool
    method setClassloader : CadmiumObj.jObject -> unit
    method setLocale : CadmiumObj.jObject -> unit
    method setResourceBundle : string -> unit
    method toString : string
    method wait : unit
    method wait'1 : int64 -> unit
    method wait'2 : int64 -> int32 -> unit
  end
